<?php
if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly
if (!class_exists('WPGP_Prayer_Admin')) {

    class WPGP_Prayer_Admin {

        public function __construct() {
            add_action('admin_enqueue_scripts', array($this, 'wpgp_enqueue_admin_scripts'));
            add_filter('manage_wpgp_prayer_request_posts_columns', array($this, 'wpgp_custom_columns'));
            add_action('manage_wpgp_prayer_request_posts_custom_column', array($this, 'wpgp_custom_column_content'), 10, 2);
            add_action('bulk_actions-edit-wpgp_prayer_request', array($this, 'wpgp_register_bulk_actions'));
            add_filter('handle_bulk_actions-edit-wpgp_prayer_request', array($this, 'wpgp_bulk_action_handler'), 10, 3);

            // Adding status column for prayer comments
            add_filter('manage_wpgp_prayer_comment_posts_columns', array($this, 'wpgp_custom_columns'));
            add_action('manage_wpgp_prayer_comment_posts_custom_column', array($this, 'wpgp_custom_column_content'), 10, 2);
            add_action('bulk_actions-edit-wpgp_prayer_comment', array($this, 'wpgp_register_bulk_actions'));
            add_filter('handle_bulk_actions-edit-wpgp_prayer_comment', array($this, 'wpgp_bulk_action_handler'), 10, 3);
			
			// Change Title placeholer , Title Label text , (no title text)
			add_filter('enter_title_here', array($this, 'wpgp_custom_title_placeholder'));
			add_filter('gettext', array($this, 'wpgp_custom_title_text'), 10, 3);
			add_filter('the_title', array($this, 'wpgp_custom_no_title_text'), 10, 2);
        }
		
		public function wpgp_custom_no_title_text($title, $post_id) {
			$post = get_post($post_id);

			if ($post->post_type == 'wpgp_prayer_request' && $title == '') {
				return __('(no request)', 'gopray');
			} elseif($post->post_type == 'wpgp_prayer_comment' && $title == ''){
				return __('(no comment)', 'gopray');
			}

			return $title;
		}
		
		public function wpgp_custom_title_text($translated_text, $text, $domain) {
            global $post_type;

            if ($post_type == 'wpgp_prayer_request' && $text == 'Title') {
                $translated_text = __('Request', 'gopray');
            } elseif($post_type == 'wpgp_prayer_comment' && $text == 'Title'){
				$translated_text = __('Comment', 'gopray');
			}

            return $translated_text;
        }
		
        public function wpgp_enqueue_admin_scripts() {
            if(function_exists('get_current_screen') ) {
                $screen = get_current_screen();
                
                if($screen->post_type == 'wpgp_prayer_comment' || $screen->post_type == 'wpgp_prayer_request' || $screen->id === 'wpgp_settings'){
                    wp_enqueue_style('prayer-admin-style', WPGP_PLUGIN_URL . 'assets/css/admin-style.css', null, WPGP_PLUGIN_VERSION);
                }
            } else {
                wp_enqueue_style('prayer-admin-style', WPGP_PLUGIN_URL . 'assets/css/admin-style.css', null, WPGP_PLUGIN_VERSION);
            }
        }

        public function wpgp_custom_columns($columns) {
			$screen = get_current_screen();
			$columns['status'] = __('Status', 'gopray');
			
			if ('wpgp_prayer_request' === $screen->post_type) {
				$columns['title'] = __('Request', 'gopray');
			} elseif('wpgp_prayer_comment' === $screen->post_type){
				$columns['title'] = __('Comment', 'gopray');
			}

            return $columns;
        }
		
		public function wpgp_custom_title_placeholder($title){
			$screen = get_current_screen();

			if ('wpgp_prayer_request' === $screen->post_type) {
				$title = __('Add Request', 'gopray');
			} elseif ('wpgp_prayer_comment' === $screen->post_type) {
				$title = __('Add Comment', 'gopray');
			}

			return $title;
		}

        public function wpgp_custom_column_content($column, $post_id) {
            if ($column == 'status') {
                if(get_post_status($post_id) == 'publish') echo esc_html(__('Approved', 'gopray'));
                else if(get_post_status($post_id) == 'pending') echo esc_html(__('Pending', 'gopray'));
                else if(get_post_status($post_id) == 'draft') echo esc_html(__('Pending', 'gopray'));
            }
        }

        public function wpgp_register_bulk_actions($bulk_actions) {
            $bulk_actions['approve'] = __('Approve', 'gopray');
            $bulk_actions['pending'] = __('Pending', 'gopray');
            return $bulk_actions;
        }

        public function wpgp_bulk_action_handler($redirect_to, $doaction, $post_ids) {
            if ($doaction === 'approve') {
                foreach ($post_ids as $post_id) {
                    wp_update_post(array('ID' => $post_id, 'post_status' => 'publish'));
                }
                $redirect_to = add_query_arg('bulk_approved', count($post_ids), $redirect_to);
            }

            if ($doaction === 'pending') {
                foreach ($post_ids as $post_id) {
                    wp_update_post(array('ID' => $post_id, 'post_status' => 'pending'));
                }
                $redirect_to = add_query_arg('bulk_pending', count($post_ids), $redirect_to);
            }

            return $redirect_to;
        }
    }

    new WPGP_Prayer_Admin();
}
