<?php
if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly
if (!class_exists('WPGP_Prayer_Comment')) {

    class WPGP_Prayer_Comment {
        
        public function __construct() {
            add_action('admin_enqueue_scripts', array($this, 'wpgp_enqueue_admin_scripts'));
            add_action('init', array($this, 'wpgp_register_post_type'));
            add_action('add_meta_boxes', array($this, 'wpgp_add_meta_boxes'));
            add_action('save_post', array($this, 'wpgp_save_meta_box_data'));
            add_action('wp_ajax_search_prayer_requests', array($this, 'wpgp_search_prayer_requests_callback'));
			add_action('before_delete_post', array($this, 'wpgp_prayer_comment_deletion'));
			add_action('trashed_post', array($this, 'wpgp_prayer_comment_deletion'));
			add_action('untrash_post', array($this, 'wpgp_prayer_comment_restore'));
        }
		
		public function wpgp_prayer_comment_restore($post_id) {
			$post_type = get_post_type($post_id);

			// Only proceed if the post type is 'wpgp_prayer_comment'
			if ($post_type !== 'wpgp_prayer_comment') {
				return;
			}

			// Fetch the related subcomments from the post meta
			$prayer_meta = get_post_meta($post_id, '_wpgp_prayer_comment_metadata', true);
			$related_comments = isset($prayer_meta['_wpgp_comment_related_sub_comments']) ? $prayer_meta['_wpgp_comment_related_sub_comments'] : [];

			// Check if related comments exist and it's an array
			if (is_array($related_comments)) {
				foreach ($related_comments as $comment_id) {
					// Check if the related comment is in the trash and has the correct post type
					if (get_post_status($comment_id) === 'trash' && get_post_type($comment_id) === 'wpgp_prayer_comment') {
						wp_untrash_post($comment_id); // Restore the subcomment
					}
				}
			}
		}
		
		
		public function wpgp_prayer_comment_deletion($post_id) {
			$post_type = get_post_type($post_id);

			// Only proceed if the post type is 'wpgp_prayer_comment'
			if ($post_type !== 'wpgp_prayer_comment') {
				return;
			}

			// Fetch the related sub comments from the post meta
			$prayer_meta = get_post_meta($post_id, '_wpgp_prayer_comment_metadata', true);
			$related_comments = $prayer_meta['_wpgp_comment_related_sub_comments'];

			// Check if related comments exist and it's an array
			if (is_array($related_comments)) {
				foreach ($related_comments as $comment_id) {
					// Check if the related comment exists
					if (get_post_type($comment_id) === 'wpgp_prayer_comment') {
						// If the prayer request post is being permanently deleted
						if (did_action('before_delete_post')) {
							wp_delete_post($comment_id, true); // true for permanent delete
						}
						// If the prayer request post is being moved to trash
						else if (did_action('trashed_post')) {
							wp_trash_post($comment_id);
						}
					}
				}
			}
		}
		
        
        public function wpgp_enqueue_admin_scripts() {
			
			$wpgp_js_variables = array(
				'wpgp_errorLoading' => __('The results could not be loaded', 'gopray'),
				'wpgp_inputTooLong_before' => __('Please delete ', 'gopray'),
				'wpgp_inputTooLong_after_singular' => __(' character', 'gopray'),
				'wpgp_inputTooLong_after_plural' => __(' characters', 'gopray'),
				'wpgp_inputTooShort_before' => __('Please enter ', 'gopray'),
				'wpgp_inputTooShort_after' => __(' or more characters', 'gopray'),
				'wpgp_loadingMore' => __('Loading more results', 'gopray'),
				'wpgp_maximumSelected_before' => __('You can only select ', 'gopray'),
				'wpgp_maximumSelected_after_singular' => __(' item', 'gopray'),
				'wpgp_maximumSelected_after_plural' => __(' items', 'gopray'),
				'wpgp_noResults' => __('No results found', 'gopray'),
				'wpgp_searching' => __('Searching', 'gopray'),
			);
			
            if(function_exists('get_current_screen') ) {
                $pt = get_current_screen()->post_type;
                
                if($pt == 'wpgp_prayer_comment'){
					wp_enqueue_script('select2', WPGP_PLUGIN_URL . 'assets/js/admin-select2.min.js', array('jquery'), WPGP_PLUGIN_VERSION, true);
					wp_localize_script( 'select2', 'wpgp_vars', $wpgp_js_variables );
					wp_enqueue_style('select2', WPGP_PLUGIN_URL . 'assets/css/admin-select2.min.css', null, WPGP_PLUGIN_VERSION);
                    wp_enqueue_script('prayer-admin-script', WPGP_PLUGIN_URL . 'assets/js/admin-prayer-comment.js', array('jquery', 'select2'), WPGP_PLUGIN_VERSION, true);                    
					
                }
				
            } else {
                wp_enqueue_script('select2', WPGP_PLUGIN_URL . 'assets/js/admin-select2.min.js', array('jquery'), WPGP_PLUGIN_VERSION, true);
				wp_localize_script( 'select2', 'wpgp_vars', $wpgp_js_variables );
				wp_enqueue_style('select2', WPGP_PLUGIN_URL . 'assets/css/admin-select2.min.css', null, WPGP_PLUGIN_VERSION);
                wp_enqueue_script('prayer-admin-script', WPGP_PLUGIN_URL . 'assets/js/admin-prayer-comment.js', array('jquery', 'select2'), WPGP_PLUGIN_VERSION, true);
            }
        }

        public static function wpgp_register_post_type() {
            $labels = array(
                'name' => __('Prayer Comments', 'gopray'),
                'singular_name' => __('Prayer Comment', 'gopray'),
                'menu_name' => __('Prayer Comments', 'gopray'),
                'name_admin_bar' => __('Prayer Comment', 'gopray'),
                'add_new' => __('Add New', 'gopray'),
                'add_new_item' => __('Add New Prayer Comment', 'gopray'),
                'new_item' => __('New Prayer Comment', 'gopray'),
                'edit_item' => __('Edit Prayer Comment', 'gopray'),
                'view_item' => __('View Prayer Comment', 'gopray'),
                'all_items' => __('Prayer Comments', 'gopray'),
                'search_items' => __('Search Prayer Comments', 'gopray'),
                'parent_item_colon' => __('Parent Prayer Comments:', 'gopray'),
                'not_found' => __('No prayer comments found', 'gopray'),
                'not_found_in_trash' => __('No prayer comments found in Trash', 'gopray')
            );

            $args = array(
                'labels' => $labels,
                'public' => true,
                'publicly_queryable' => false,
                'show_ui' => true,
                'show_in_menu' => 'wpgp_plugin_menu',
                'query_var' => true,
                'capability_type' => 'post',
                'has_archive' => false,
                'hierarchical' => false,
                'menu_position' => null,
                'supports' => array('title'),
            );

            register_post_type('wpgp_prayer_comment', $args);
        }

        public static function wpgp_add_meta_boxes() {
            add_meta_box(
                'wpgp_prayer_comment_meta_box',
                __('Prayer Comment Details', 'gopray'),
                array(__CLASS__, 'wpgp_render_meta_box'),
                'wpgp_prayer_comment',
                'advanced',
                'high'
            );
        }

        public static function wpgp_render_meta_box($post) {
            wp_nonce_field('wpgp_prayer_comment_save_meta_box_data', 'wpgp_prayer_comment_meta_box_nonce');
            
			$meta_data = get_post_meta($post->ID, '_wpgp_prayer_comment_metadata', true);
			
			$name = isset($meta_data['_wpgp_prayer_comment_name']) ? $meta_data['_wpgp_prayer_comment_name'] : '';
			$email = isset($meta_data['_wpgp_prayer_comment_email']) ? $meta_data['_wpgp_prayer_comment_email'] : '';
			$date = isset($meta_data['_wpgp_prayer_comment_date']) ? $meta_data['_wpgp_prayer_comment_date'] : '';
			$ip_address = isset($meta_data['_wpgp_prayer_comment_ip_address']) ? $meta_data['_wpgp_prayer_comment_ip_address'] : '';
			$likes = isset($meta_data['_wpgp_prayer_comment_likes']) ? $meta_data['_wpgp_prayer_comment_likes'] : '';
			$parent_request = isset($meta_data['_wpgp_parent_request_id']) ? $meta_data['_wpgp_parent_request_id'] : '';
			$related_sub_comments_array = isset($meta_data['_wpgp_comment_related_sub_comments']) && is_array($meta_data['_wpgp_comment_related_sub_comments']) ? $meta_data['_wpgp_comment_related_sub_comments'] : [];

			echo '<table class="form-table wpgp-fields-table">';
			echo '<tr>';
			echo '<td><label for="_wpgp_prayer_comment_name"><strong>' . esc_html__('Name', 'gopray') . '</strong></label></td>';
			echo '<td><input type="text" id="wpgp_prayer_comment_name" name="_wpgp_prayer_comment_metadata[_wpgp_prayer_comment_name]" value="' . esc_attr($name) . '" /></td>';
			echo '</tr>';

			echo '<tr>';
			echo '<td><label for="_wpgp_prayer_comment_email"><strong>' . esc_html__('Email', 'gopray') . '</strong></label></td>';
			echo '<td><input type="email" id="wpgp_prayer_comment_email" name="_wpgp_prayer_comment_metadata[_wpgp_prayer_comment_email]" value="' . esc_attr($email) . '" size="25" /></td>';
			echo '</tr>';

			echo '<tr>';
			echo '<td><label for="_wpgp_prayer_comment_date"><strong>' . esc_html__('Date Received', 'gopray') . '</strong></label></td>';
			echo '<td><input type="date" id="wpgp_prayer_comment_date" name="_wpgp_prayer_comment_metadata[_wpgp_prayer_comment_date]" value="' . esc_attr($date) . '" size="25" /></td>';
			echo '</tr>';

			echo '<tr>';
			echo '<td><label for="_wpgp_prayer_comment_ip_address"><strong>' . esc_html__('IP Address', 'gopray') . '</strong></label></td>';
			echo '<td><input type="text" id="wpgp_prayer_comment_ip_address" name="_wpgp_prayer_comment_metadata[_wpgp_prayer_comment_ip_address]" value="' . esc_attr($ip_address) . '" size="25" /></td>';
			echo '</tr>';

			echo '<tr>';
			echo '<td><label for="_wpgp_prayer_comment_likes"><strong>' . esc_html__('Likes', 'gopray') . '</strong></label></td>';
			echo '<td><input type="text" id="wpgp_prayer_comment_likes" name="_wpgp_prayer_comment_metadata[_wpgp_prayer_comment_likes]" value="' . esc_attr($likes) . '" size="25" /></td>';
			echo '</tr>';

			echo '<tr lang="en">';
			echo '<td><label for="_wpgp_parent_request_id"><strong>' . esc_html__('Related Prayer Request', 'gopray') . '</strong></label></td>';
			echo '<td lang="en"><select id="wpgp_parent_request" name="_wpgp_prayer_comment_metadata[_wpgp_parent_request_id]" style="width: 100%;" data-placeholder="' . esc_html__('Select a prayer request', 'gopray') . '" lang="en">';
			if ($parent_request) {
				$post_title = get_the_title($parent_request);
				echo '<option value="' . esc_attr($parent_request) . '" selected>' . esc_html($post_title) . '</option>';
			}
			echo '</select></td>';
			echo '</tr>';
			
			if($related_sub_comments_array) {
				$count = 0;

				foreach($related_sub_comments_array as $related_sub_comment) {
					if(get_post_status($related_sub_comment) == 'trash') continue;
					$count++;
					$pending_label = get_post_status($related_sub_comment) != 'publish' ? ' - ' . esc_html__('Pending', 'gopray') . ' ' : '';
					if($count == 1){
						echo '<tr>';
						echo '<td colspan="2"><h3 style="margin-bottom: 0;">' . esc_html__('Related Sub Comments', 'gopray') . '</h3></td>';
						echo '</tr>';
					}
					echo '<tr>';
					echo '<td colspan="2">' . esc_html(get_the_title($related_sub_comment)) . ' (<a href="' . esc_url(get_edit_post_link($related_sub_comment)) . '">' . esc_html__('edit comment', 'gopray') . '</a>)' . esc_html($pending_label) . '</td>';
					echo '</tr>';
				}
			}
			
			echo '</table>';
        }

        public static function wpgp_save_meta_box_data($post_id) {
            if (!isset($_POST['wpgp_prayer_comment_meta_box_nonce'])) {
                return;
            }

            if (!wp_verify_nonce(sanitize_text_field( wp_unslash($_POST['wpgp_prayer_comment_meta_box_nonce']) ), 'wpgp_prayer_comment_save_meta_box_data')) {
                return;
            }

            if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
                return;
            }
            
            if (!current_user_can('edit_post', $post_id)) {
                return;
            }
            
            $_wpgp_prayer_comment_metadata = array();
            
            $fields = array(
				'_wpgp_prayer_comment_name',
				'_wpgp_prayer_comment_email',
				'_wpgp_prayer_comment_date',
				'_wpgp_prayer_comment_ip_address',
				'_wpgp_prayer_comment_likes',
				'_wpgp_parent_request_id'
			);

			// Check if the form was submitted and process the data
			if (isset($_POST['_wpgp_prayer_comment_metadata'])) {	
				
				$meta_data = get_post_meta($post_id, '_wpgp_prayer_comment_metadata', true);
				$old_meta_data = $meta_data;

				// Ensure $meta_data is an array
				if (!is_array($meta_data)) {
					$meta_data = array();
				}

				// Sanitize and save the fields
				foreach ($fields as $field) {
					if (isset($_POST['_wpgp_prayer_comment_metadata'][$field])) {
						$meta_data[$field] = sanitize_text_field( wp_unslash($_POST['_wpgp_prayer_comment_metadata'][$field]) );
					} else {
						$meta_data[$field] = '';
					}
				}
				
				// Handle the parent request update
				$old_parent_request_id = isset($old_meta_data['_wpgp_parent_request_id']) ? $old_meta_data['_wpgp_parent_request_id'] : '';

				// Save the updated meta data
				update_post_meta($post_id, '_wpgp_prayer_comment_metadata', $meta_data);

				if ( (isset($_POST['_wpgp_prayer_comment_metadata']['_wpgp_parent_request_id'])) && ($_POST['_wpgp_prayer_comment_metadata']['_wpgp_parent_request_id'] != $old_parent_request_id) ) {
					// Remove comment from old parent request
					if ($old_parent_request_id && get_post_type($old_parent_request_id) == 'wpgp_prayer_request' ) {
						$old_parent_metadata = get_post_meta($old_parent_request_id, '_wpgp_prayer_request_metadata', true);

						if (!isset($old_parent_metadata['_wpgp_prayer_request_related_comments']) || !is_array($old_parent_metadata['_wpgp_prayer_request_related_comments'])) {
							$old_parent_metadata['_wpgp_prayer_request_related_comments'] = array();
						}

						if (in_array($post_id, $old_parent_metadata['_wpgp_prayer_request_related_comments'])) {
							$index = array_search($post_id, $old_parent_metadata['_wpgp_prayer_request_related_comments']);
							unset($old_parent_metadata['_wpgp_prayer_request_related_comments'][$index]);
							update_post_meta($old_parent_request_id, '_wpgp_prayer_request_metadata', $old_parent_metadata);
						}
					} elseif($old_parent_request_id && get_post_type($old_parent_request_id) == 'wpgp_prayer_comment' ) {
						$old_parent_metadata = get_post_meta($old_parent_request_id, '_wpgp_prayer_comment_metadata', true);

						if (!isset($old_parent_metadata['_wpgp_comment_related_sub_comments']) || !is_array($old_parent_metadata['_wpgp_comment_related_sub_comments'])) {
							$old_parent_metadata['_wpgp_comment_related_sub_comments'] = array();
						}

						if (in_array($post_id, $old_parent_metadata['_wpgp_comment_related_sub_comments'])) {
							$index = array_search($post_id, $old_parent_metadata['_wpgp_comment_related_sub_comments']);
							unset($old_parent_metadata['_wpgp_comment_related_sub_comments'][$index]);
							update_post_meta($old_parent_request_id, '_wpgp_prayer_comment_metadata', $old_parent_metadata);
						}
					}

					// Add comment to new parent request
					$parent_request_id = sanitize_text_field( wp_unslash($_POST['_wpgp_prayer_comment_metadata']['_wpgp_parent_request_id']) );
					
					if(get_post_type($parent_request_id) == 'wpgp_prayer_request' ){
						
						$parent_metadata = get_post_meta($parent_request_id, '_wpgp_prayer_request_metadata', true);
						
						if (!isset($parent_metadata['_wpgp_prayer_request_related_comments']) || !is_array($parent_metadata['_wpgp_prayer_request_related_comments'])) {
							$parent_metadata['_wpgp_prayer_request_related_comments'] = array();
						}

						if (!in_array($post_id, $parent_metadata['_wpgp_prayer_request_related_comments'])) {
							$parent_metadata['_wpgp_prayer_request_related_comments'][] = $post_id;
							update_post_meta($parent_request_id, '_wpgp_prayer_request_metadata', $parent_metadata);
						}
						
					} elseif(get_post_type($parent_request_id) == 'wpgp_prayer_comment' ){
						
						$parent_metadata = get_post_meta($parent_request_id, '_wpgp_prayer_comment_metadata', true);
						
						if (!isset($parent_metadata['_wpgp_comment_related_sub_comments']) || !is_array($parent_metadata['_wpgp_comment_related_sub_comments'])) {
							$parent_metadata['_wpgp_comment_related_sub_comments'] = array();
						}

						if (!in_array($post_id, $parent_metadata['_wpgp_comment_related_sub_comments'])) {
							$parent_metadata['_wpgp_comment_related_sub_comments'][] = $post_id;
							update_post_meta($parent_request_id, '_wpgp_prayer_comment_metadata', $parent_metadata);
						}
						
					}
					
				}
			}
        }

        public static function wpgp_search_prayer_requests_callback() {
            
            if (!isset($_GET['security']) || !wp_verify_nonce(sanitize_text_field( wp_unslash($_GET['security']) ), 'wpgp_prayer_comment_save_meta_box_data')) {
                return;
            }
        
            if (!current_user_can('edit_posts')) {
                wp_send_json_error('Unauthorized', 403);
                return;
            }
        
			$search_term = isset($_GET['q']) ? sanitize_text_field( wp_unslash($_GET['q']) ) : '';
			
            $query = new WP_Query(array(
                'post_type' => array( 'wpgp_prayer_request', 'wpgp_prayer_comment' ),
                's' => $search_term,
                'posts_per_page' => 10,
				'post_status' 	 => array('publish'),
            ));
        
            $results = array();
            if ($query->have_posts()) {
                while ($query->have_posts()) {
                    $query->the_post();
					$title = get_the_title();
					$title = html_entity_decode($title, ENT_QUOTES, 'UTF-8');
                    $results[] = array(
                        'id' => get_the_ID(),
                        'text' => esc_html($title)
                    );
                }
                wp_reset_postdata();
            }
        
            wp_send_json($results);
        }
    }
    
    new WPGP_Prayer_Comment();
    
}
