<?php
if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly
if (!class_exists('WPGP_Prayer_Request')) {

    class WPGP_Prayer_Request {
        
        public function __construct() {
            add_action('init', array($this, 'wpgp_register_post_type'));
            add_action('add_meta_boxes', array($this, 'wpgp_add_meta_boxes'));
            add_action('save_post', array($this, 'wpgp_save_meta_box_data'));
            add_action('admin_menu', array($this, 'wpgp_plugin_menu'));
			add_action('before_delete_post', array($this, 'wpgp_prayer_request_deletion'));
			add_action('trashed_post', array($this, 'wpgp_prayer_request_deletion'));
			add_action('untrash_post', array($this, 'wpgp_prayer_request_restore'));
        }
		
		public function wpgp_prayer_request_restore($post_id) {
			$post_type = get_post_type($post_id);

			// Only proceed if the post type is 'wpgp_prayer_request'
			if ($post_type !== 'wpgp_prayer_request') {
				return;
			}

			// Fetch the related comments from the post meta
			$prayer_meta = get_post_meta($post_id, '_wpgp_prayer_request_metadata', true);
			$related_comments = $prayer_meta['_wpgp_prayer_request_related_comments'];

			// Check if related comments exist and it's an array
			if (is_array($related_comments)) {
				foreach ($related_comments as $comment_id) {
					// Check if the related comment exists and is in the trash
					if (get_post_status($comment_id) === 'trash' && get_post_type($comment_id) === 'wpgp_prayer_comment') {
						wp_untrash_post($comment_id); // Restore the comment
					}
				}
			}
		}
		
		
		public function wpgp_prayer_request_deletion($post_id) {
			$post_type = get_post_type($post_id);

			// Only proceed if the post type is 'wpgp_prayer_request'
			if ($post_type !== 'wpgp_prayer_request') {
				return;
			}

			// Fetch the related comments from the post meta
			$prayer_meta = get_post_meta($post_id, '_wpgp_prayer_request_metadata', true);
			$related_comments = $prayer_meta['_wpgp_prayer_request_related_comments'];

			// Check if related comments exist and it's an array
			if (is_array($related_comments)) {
				foreach ($related_comments as $comment_id) {
					// Check if the related comment exists
					if (get_post_type($comment_id) === 'wpgp_prayer_comment') {
						// If the prayer request post is being permanently deleted
						if (did_action('before_delete_post')) {
							wp_delete_post($comment_id, true); // true for permanent delete
						}
						// If the prayer request post is being moved to trash
						else if (did_action('trashed_post')) {
							wp_trash_post($comment_id);
						}
					}
				}
			}
		}
		

        public static function wpgp_register_post_type() {
            $labels = array(
                'name' => __('Prayer Requests', 'gopray'),
                'singular_name' => __('Prayer Request', 'gopray'),
                'menu_name' => __('Prayer Requests', 'gopray'),
                'name_admin_bar' => __('Prayer Request', 'gopray'),
                'add_new' => __('Add New', 'gopray'),
                'add_new_item' => __('Add New Prayer Request', 'gopray'),
                'new_item' => __('New Prayer Request', 'gopray'),
                'edit_item' => __('Edit Prayer Request', 'gopray'),
                'view_item' => __('View Prayer Request', 'gopray'),
                'all_items' => __('Prayer Requests', 'gopray'),
                'search_items' => __('Search Prayer Requests', 'gopray'),
                'parent_item_colon' => __('Parent Prayer Requests:', 'gopray'),
                'not_found' => __('No prayer requests found', 'gopray'),
                'not_found_in_trash' => __('No prayer requests found in Trash', 'gopray')
            );

            $args = array(
                'labels' => $labels,
                'public' => true,
                'publicly_queryable' => false,
                'show_ui' => true,
                'show_in_menu' => 'wpgp_plugin_menu',
                'query_var' => true,
                'capability_type' => 'post',
                'has_archive' => false,
                'hierarchical' => false,
                'menu_position' => null,
                'supports' => array('title'),
            );

            register_post_type('wpgp_prayer_request', $args);
        }

        public static function wpgp_add_meta_boxes() {
            add_meta_box(
                'wpgp_prayer_request_meta_box',
                __('Prayer Request Details', 'gopray'),
                array(__CLASS__, 'wpgp_render_meta_box'),
                'wpgp_prayer_request',
                'advanced',
                'high'
            );
        }

        public static function wpgp_render_meta_box($post) {
            wp_nonce_field('wpgp_prayer_request_save_meta_box_data', 'wpgp_prayer_request_meta_box_nonce');
            
            $meta_data = get_post_meta($post->ID, '_wpgp_prayer_request_metadata', true);

			$name = isset($meta_data['_wpgp_prayer_request_name']) ? $meta_data['_wpgp_prayer_request_name'] : '';
			$email = isset($meta_data['_wpgp_prayer_request_email']) ? $meta_data['_wpgp_prayer_request_email'] : '';
			$date = isset($meta_data['_wpgp_prayer_request_date']) ? $meta_data['_wpgp_prayer_request_date'] : '';
			$ip_address = isset($meta_data['_wpgp_prayer_request_ip_address']) ? $meta_data['_wpgp_prayer_request_ip_address'] : '';
			$likes = isset($meta_data['_wpgp_prayer_request_likes']) ? $meta_data['_wpgp_prayer_request_likes'] : '';
			$related_comments_array = isset($meta_data['_wpgp_prayer_request_related_comments']) && is_array($meta_data['_wpgp_prayer_request_related_comments']) ? $meta_data['_wpgp_prayer_request_related_comments'] : [];

			echo '<table class="form-table wpgp-fields-table">';
			echo '<tr>';
			echo '<td><label for="_wpgp_prayer_request_name"><strong>' . esc_html__('Name', 'gopray') . '</strong></label></td>';
			echo '<td><input type="text" id="wpgp_prayer_request_name" name="_wpgp_prayer_request_metadata[_wpgp_prayer_request_name]" value="' . esc_attr($name) . '" size="25" /></td>';
			echo '</tr>';

			echo '<tr>';
			echo '<td><label for="_wpgp_prayer_request_email"><strong>' . esc_html__('Email', 'gopray') . '</strong></label></td>';
			echo '<td><input type="email" id="wpgp_prayer_request_email" name="_wpgp_prayer_request_metadata[_wpgp_prayer_request_email]" value="' . esc_attr($email) . '" size="25" /></td>';
			echo '</tr>';

			echo '<tr>';
			echo '<td><label for="_wpgp_prayer_request_date"><strong>' . esc_html__('Date Received', 'gopray') . '</strong></label></td>';
			echo '<td><input type="date" id="wpgp_prayer_request_date" name="_wpgp_prayer_request_metadata[_wpgp_prayer_request_date]" value="' . esc_attr($date) . '" size="25" /></td>';
			echo '</tr>';

			echo '<tr>';
			echo '<td><label for="_wpgp_prayer_request_ip_address"><strong>' . esc_html__('IP Address', 'gopray') . '</strong></label></td>';
			echo '<td><input type="text" id="wpgp_prayer_request_ip_address" name="_wpgp_prayer_request_metadata[_wpgp_prayer_request_ip_address]" value="' . esc_attr($ip_address) . '" size="25" /></td>';
			echo '</tr>';

			echo '<tr>';
			echo '<td><label for="_wpgp_prayer_request_likes"><strong>' . esc_html__('Likes', 'gopray') . '</strong></label></td>';
			echo '<td><input type="text" id="wpgp_prayer_request_likes" name="_wpgp_prayer_request_metadata[_wpgp_prayer_request_likes]" value="' . esc_attr($likes) . '" size="25" /></td>';
			echo '</tr>';
            
            if($related_comments_array) {
				$count = 0;
				
				foreach($related_comments_array as $related_comment) {
					if(get_post_status($related_comment) == 'trash') continue;
					$count++;
					$pending_label = get_post_status($related_comment) != 'publish' ? ' - ' . esc_html__('Pending', 'gopray') . ' ' : '';
					if($count == 1){
						echo '<tr>';
						echo '<td colspan="2"><h3 style="margin-bottom: 0;">' . esc_html__('Related Comments', 'gopray') . '</h3></td>';
						echo '</tr>';
					}
					echo '<tr>';
					echo '<td colspan="2">' . esc_html(get_the_title($related_comment)) . ' (<a href="' . esc_url(get_edit_post_link($related_comment)) . '">' . esc_html__('edit comment', 'gopray') . '</a>)' . esc_html($pending_label) . '</td>';
					echo '</tr>';
				}
			}

			echo '</table>';
        }

        public static function wpgp_save_meta_box_data($post_id) {
            if (!isset($_POST['wpgp_prayer_request_meta_box_nonce'])) {
                return;
            }

			$wpgp_prayer_request_meta_box_nonce = sanitize_text_field( wp_unslash($_POST['wpgp_prayer_request_meta_box_nonce']) );
				
            if (!wp_verify_nonce($wpgp_prayer_request_meta_box_nonce, 'wpgp_prayer_request_save_meta_box_data')) {
                return;
            }

            if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
                return;
            }

            if (!current_user_can('edit_post', $post_id)) {
                return;
            }
            
            $fields = array(
                '_wpgp_prayer_request_name',
                '_wpgp_prayer_request_email',
                '_wpgp_prayer_request_date',
                '_wpgp_prayer_request_ip_address',
                '_wpgp_prayer_request_likes',
            );
            
            if (isset($_POST['_wpgp_prayer_request_metadata'])) {
				$meta_data = get_post_meta($post_id, '_wpgp_prayer_request_metadata', true);

				if (!$meta_data) {
					$meta_data = array();
				}

				foreach ($fields as $field) {
					if (isset($_POST['_wpgp_prayer_request_metadata'][$field])) {
						$meta_data[$field] = sanitize_text_field( wp_unslash($_POST['_wpgp_prayer_request_metadata'][$field]) );
					} else {
						$meta_data[$field] = '';
					}
				}
				
				if (!isset($meta_data['_wpgp_prayer_request_related_comments'])) {
					$meta_data['_wpgp_prayer_request_related_comments'] = array();
				}

				update_post_meta($post_id, '_wpgp_prayer_request_metadata', $meta_data);
			}
		}
        
        public static function wpgp_plugin_menu() {
            add_menu_page(
                __('GoPray', 'gopray'), 
                __('GoPray', 'gopray'), 
                'manage_options', 
                'wpgp_plugin_menu', 
                '', 
                'dashicons-admin-comments', 
                100
            );
        }
    }
    
    new WPGP_Prayer_Request();

}