<?php
if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly
if (!class_exists('WPGP_Prayer_Settings')) {

    class WPGP_Prayer_Settings {

        public function __construct() {
            add_action('admin_init', array($this, 'wpgp_settings_init'));
            add_action('admin_menu', array($this, 'wpgp_add_admin_menu'));
			add_action('admin_enqueue_scripts', array($this, 'wpgp_enqueue_admin_scripts'));
        }
		
		public function wpgp_enqueue_admin_scripts() {
			 if(function_exists('get_current_screen') ) {
                $screen = get_current_screen();
                if(isset($screen->id) && $screen->id === 'gopray_page_wpgp_settings'){
                    wp_enqueue_style('wp-color-picker');
					wp_enqueue_script('wp-color-picker');
                    wp_enqueue_script('prayer-admin-script', WPGP_PLUGIN_URL . 'assets/js/admin-settings.js', array('jquery', 'wp-color-picker'), WPGP_PLUGIN_VERSION, true);
                }
            } else {
                wp_enqueue_style('wp-color-picker');
				wp_enqueue_script('wp-color-picker');
                wp_enqueue_script('prayer-admin-script', WPGP_PLUGIN_URL . 'assets/js/admin-settings.js', array('jquery', 'wp-color-picker'), WPGP_PLUGIN_VERSION, true);
			 }
        }
		
		public function wpgp_add_admin_menu() {
            add_submenu_page(
                'wpgp_plugin_menu',
                __('GoPray Settings', 'gopray'),
                __('Settings', 'gopray'),
                'manage_options',
                'wpgp_settings',
                array($this, 'wpgp_settings_page')
            );
        }

        public function wpgp_settings_page() {
            settings_errors();
            ?>
            <div class="wrap">
                <h1><?php echo esc_html__('GoPray Settings', 'gopray'); ?></h1>
                <form action="options.php" method="post">
                    <?php
                    settings_fields('wpgp_settings');
                    do_settings_sections('gopray');
                    submit_button();
					wp_nonce_field('wpgp_save_settings', 'wpgp_nonce');
                    ?>
                </form>
            </div>
            <?php
        }

        public function wpgp_settings_init() {
            register_setting('wpgp_settings', 'wpgp_settings_options', array($this, 'wpgp_sanitize_options'));

            add_settings_section(
				'wpgp_request_form_options', 
				__('Prayer Request Form Settings', 'gopray'), 
				array($this, 'wpgp_request_form_options_callback'), 
				'gopray'
			);
			
			add_settings_section(
				'wpgp_preyer_wall_options', 
				__('Prayer Wall Settings', 'gopray'), 
				array($this, 'wpgp_section_preyer_wall_callback'), 
				'gopray'
			);

			// Add fields related to prayer form
			add_settings_field(
				'wpgp_name_required', 
				__('Name field is required', 'gopray'), 
				array($this, 'wpgp_name_required_render'), 
				'gopray', 
				'wpgp_request_form_options'
			);
			
			add_settings_field(
				'wpgp_email_required', 
				__('Email field is required', 'gopray'), 
				array($this, 'wpgp_email_required_render'), 
				'gopray', 
				'wpgp_request_form_options'
			);

            add_settings_field(
				'wpgp_email_notifications_user', 
				__('Email notification to user once the prayer request is submitted', 'gopray'), 
				array($this, 'wpgp_email_notifications_user_render'), 
				'gopray', 
				'wpgp_request_form_options'
			);
			
			add_settings_field(
				'wpgp_email_notifications_template_user', 
				__('Email notifications template (User)', 'gopray'), 
				array($this, 'wpgp_email_notifications_template_user_render'), 
				'gopray', 
				'wpgp_request_form_options'
			);

			add_settings_field(
				'wpgp_email_notifications_admin', 
				__('Email notifications to admin once a new prayer request is submitted', 'gopray'), 
				array($this, 'wpgp_email_notifications_admin_render'), 
				'gopray', 
				'wpgp_request_form_options'
			);
			
			add_settings_field(
				'wpgp_email_notifications_template_admin', 
				__('Email notifications template (Admin)', 'gopray'), 
				array($this, 'wpgp_email_notifications_template_admin_render'), 
				'gopray', 
				'wpgp_request_form_options'
			);

			add_settings_field(
				'wpgp_enable_captcha', 
				__('Enable Google reCAPTCHA', 'gopray'), 
				array($this, 'wpgp_enable_captcha_render'), 
				'gopray', 
				'wpgp_request_form_options'
			);
			
			add_settings_field(
				'wpgp_recaptcha_site_key_v2', 
				__('Site Key (reCAPTCHA v2)', 'gopray'), 
				array($this, 'wpgp_recaptcha_site_key_v2_render'), 
				'gopray', 
				'wpgp_request_form_options'
			);

			add_settings_field(
				'wpgp_recaptcha_secret_key_v2', 
				__('Secret Key (reCAPTCHA v2)', 'gopray'), 
				array($this, 'wpgp_recaptcha_secret_key_v2_render'), 
				'gopray', 
				'wpgp_request_form_options'
			);
			
			add_settings_field(
				'wpgp_recaptcha_site_key_v3', 
				__('Site Key (reCAPTCHA v3)', 'gopray'), 
				array($this, 'wpgp_recaptcha_site_key_v3_render'), 
				'gopray', 
				'wpgp_request_form_options'
			);

			add_settings_field(
				'wpgp_recaptcha_secret_key_v3', 
				__('Secret Key  (reCAPTCHA v3)', 'gopray'), 
				array($this, 'wpgp_recaptcha_secret_key_v3_render'), 
				'gopray', 
				'wpgp_request_form_options'
			);

			add_settings_field(
				'wpgp_restrict_access', 
				__('Restrict form access to logged in users', 'gopray'), 
				array($this, 'wpgp_restrict_access_render'), 
				'gopray', 
				'wpgp_request_form_options'
			);
			
			add_settings_field(
				'wpgp_submit_color', 
				__('Submit button color', 'gopray'), 
				array($this, 'wpgp_submit_color_render'), 
				'gopray', 
				'wpgp_request_form_options'
			);
			
			add_settings_field(
				'wpgp_mail_with_smtp', 
				__('Are you using a SMTP plugin like WP Mail SMTP to send emails?', 'gopray'), 
				array($this, 'wpgp_mail_with_smtp_render'), 
				'gopray', 
				'wpgp_request_form_options'
			);


            // Add fields related to prayer wall

			add_settings_field(
				'wpgp_enable_show_date', 
				__('Enable prayer request/comments date', 'gopray'), 
				array($this, 'wpgp_enable_show_date_render'), 
				'gopray', 
				'wpgp_preyer_wall_options'
			);
			
            add_settings_field(
				'wpgp_enable_comments', 
				__('Enable user comments', 'gopray'), 
				array($this, 'wpgp_enable_comments_render'), 
				'gopray', 
				'wpgp_preyer_wall_options'
			);

            add_settings_field(
				'wpgp_enable_hearts', 
				__('Enable like button', 'gopray'), 
				array($this, 'wpgp_enable_hearts_render'), 
				'gopray', 
				'wpgp_preyer_wall_options'
			);
			
			add_settings_field(
				'wpgp_display_period', 
				__('Display prayer requests', 'gopray'), 
				array($this, 'wpgp_display_period_render'), 
				'gopray', 
				'wpgp_preyer_wall_options'
			);
			
			add_settings_field(
				'wpgp_reply_color', 
				__('Reply button color', 'gopray'), 
				array($this, 'wpgp_reply_color_render'), 
				'gopray', 
				'wpgp_preyer_wall_options'
			);

            add_settings_field(
				'wpgp_load_color', 
				__('Load more button color', 'gopray'), 
				array($this, 'wpgp_load_color_render'), 
				'gopray', 
				'wpgp_preyer_wall_options'
			);

        }
		
		public function wpgp_sanitize_options($options) {

		    if ( !isset($_POST['wpgp_nonce']) || !wp_verify_nonce( sanitize_text_field( wp_unslash($_POST['wpgp_nonce']) ), 'wpgp_save_settings' ) ) {
                add_settings_error('wpgp_settings_options', 'wpgp_nonce_error', __('Nonce verification failed', 'gopray'));
                return get_option('wpgp_settings_options');
            }
			
			$options['wpgp_name_field_required'] = isset($options['wpgp_name_field_required']) ? absint($options['wpgp_name_field_required']) : $options['wpgp_name_field_required'];
            
            $options['wpgp_email_field_required'] = isset($options['wpgp_email_field_required']) ? absint($options['wpgp_email_field_required']) : $options['wpgp_email_field_required'];
			
			$options['wpgp_email_notifications_user'] = isset($options['wpgp_email_notifications_user']) ? absint($options['wpgp_email_notifications_user']) : $options['wpgp_email_notifications_user'];
            
            $options['wpgp_email_notifications_template_user'] = isset($options['wpgp_email_notifications_template_user']) ? wp_kses_post($options['wpgp_email_notifications_template_user']) : $options['wpgp_email_notifications_template_user'];
            
            $options['wpgp_email_notifications_admin'] = isset($options['wpgp_email_notifications_admin']) ? absint($options['wpgp_email_notifications_admin']) : $options['wpgp_email_notifications_admin'];
			
			$options['wpgp_email_notifications_template_admin'] = isset($options['wpgp_email_notifications_template_admin']) ? wp_kses_post($options['wpgp_email_notifications_template_admin']) : $options['wpgp_email_notifications_template_admin'];
			
			$options['wpgp_mail_with_smtp'] = isset($options['wpgp_mail_with_smtp']) ? absint($options['wpgp_mail_with_smtp']) : $options['wpgp_mail_with_smtp'];
			
			$options['wpgp_recaptcha_type'] = isset($options['wpgp_recaptcha_type']) ? sanitize_text_field($options['wpgp_recaptcha_type']) : $options['wpgp_recaptcha_type'];
            
            $options['wpgp_recaptcha_site_key_v2'] = isset($options['wpgp_recaptcha_site_key_v2']) ? sanitize_text_field($options['wpgp_recaptcha_site_key_v2']) : $options['wpgp_recaptcha_site_key_v2'];
            
            $options['wpgp_recaptcha_secret_key_v2'] = isset($options['wpgp_recaptcha_secret_key_v2']) ? sanitize_text_field($options['wpgp_recaptcha_secret_key_v2']) : $options['wpgp_recaptcha_secret_key_v2'];
            
            $options['wpgp_recaptcha_site_key_v3'] = isset($options['wpgp_recaptcha_site_key_v3']) ? sanitize_text_field($options['wpgp_recaptcha_site_key_v3']) : $options['wpgp_recaptcha_site_key_v3'];
            
            $options['wpgp_recaptcha_secret_key_v3'] = isset($options['wpgp_recaptcha_secret_key_v3']) ? sanitize_text_field($options['wpgp_recaptcha_secret_key_v3']) : $options['wpgp_recaptcha_secret_key_v3'];
			
			$options['wpgp_restrict_access'] = isset($options['wpgp_restrict_access']) ? absint($options['wpgp_restrict_access']) : $options['wpgp_restrict_access'];
            
            $options['wpgp_send_button_color'] = isset($options['wpgp_send_button_color']) ? sanitize_hex_color($options['wpgp_send_button_color']) : $options['wpgp_send_button_color'];
            
			$options['wpgp_enable_show_date'] = isset($options['wpgp_enable_show_date']) ? absint($options['wpgp_enable_show_date']) : $options['wpgp_enable_show_date'];
				
            $options['wpgp_enable_comments'] = isset($options['wpgp_enable_comments']) ? absint($options['wpgp_enable_comments']) : $options['wpgp_enable_comments'];
            
            $options['wpgp_enable_hearts'] = isset($options['wpgp_enable_hearts']) ? absint($options['wpgp_enable_hearts']) : $options['wpgp_enable_hearts'];
           
            $options['wpgp_display_period'] = isset($options['wpgp_display_period']) ? sanitize_text_field($options['wpgp_display_period']) : $options['wpgp_display_period'];
            
            $options['wpgp_reply_button_color'] = isset($options['wpgp_reply_button_color']) ? sanitize_hex_color($options['wpgp_reply_button_color']) : $options['wpgp_reply_button_color'];
			            
            $options['wpgp_load_more_button_color'] = isset($options['wpgp_load_more_button_color']) ? sanitize_hex_color($options['wpgp_load_more_button_color']) : $options['wpgp_load_more_button_color'];
        
            return $options;
		}
		
		public function wpgp_request_form_options_callback() { ?>
			<p class="wpgp-shortcode-details"><?php echo esc_html__('Please use', 'gopray'); ?> <strong>[wpgp_prayer_request_form]</strong> <?php echo esc_html__('shortcode to add the prayer request form on any post/page', 'gopray'); ?></p>
		<?php
		}
		
		public function wpgp_section_preyer_wall_callback() { ?>
			<p class="wpgp-shortcode-details"><?php echo esc_html__('Please use', 'gopray'); ?> <strong>[wpgp_display_prayer_wall]</strong> <?php echo esc_html__('shortcode to display the prayer wall on any post/page', 'gopray'); ?></p>
		<?php
		}
		
		public function wpgp_name_required_render() {
			$options = get_option('wpgp_settings_options', array());
			?>
			<input type='checkbox' name='wpgp_settings_options[wpgp_name_field_required]' <?php checked(isset($options['wpgp_name_field_required']) ? $options['wpgp_name_field_required'] : 0, 1); ?> value='1'>
			<?php
		}
		
		public function wpgp_email_required_render() {
			$options = get_option('wpgp_settings_options', array());
			?>
			<input type='checkbox' name='wpgp_settings_options[wpgp_email_field_required]' <?php checked(isset($options['wpgp_email_field_required']) ? $options['wpgp_email_field_required'] : 0, 1); ?> value='1'>
			<?php
		}
		
		public function wpgp_email_notifications_user_render() {
			$options = get_option('wpgp_settings_options', array());
			?>
			<input type='checkbox' id='wpgp_email_notifications_user' name='wpgp_settings_options[wpgp_email_notifications_user]' <?php checked(isset($options['wpgp_email_notifications_user']) ? $options['wpgp_email_notifications_user'] : 0, 1); ?> value='1'>
			<?php
		}
		
		public function wpgp_email_notifications_template_user_render() {
			$options = get_option('wpgp_settings_options', array());
			$email_notifications_template_user = isset($options['wpgp_email_notifications_template_user']) ? $options['wpgp_email_notifications_template_user'] : '';
			
			$editor_settings = array(
				'textarea_name' => 'wpgp_settings_options[wpgp_email_notifications_template_user]',
				'media_buttons' => true,
				'textarea_rows' => 10,
				'tinymce'       => true,
				'quicktags'     => true,
			);

			// Display the editor
			wp_editor($email_notifications_template_user, 'wpgp_email_notifications_template_user', $editor_settings);
		}

		public function wpgp_email_notifications_admin_render() {
			$options = get_option('wpgp_settings_options', array());
			?>
			<input type='checkbox' id='wpgp_email_notifications_admin'  name='wpgp_settings_options[wpgp_email_notifications_admin]' <?php checked(isset($options['wpgp_email_notifications_admin']) ? $options['wpgp_email_notifications_admin'] : 0, 1); ?> value='1'>
			<?php
		}
		
		public function wpgp_email_notifications_template_admin_render() {
			$options = get_option('wpgp_settings_options', array());
			$email_notifications_template_user = isset($options['wpgp_email_notifications_template_admin']) ? $options['wpgp_email_notifications_template_admin'] : '';
			
			$editor_settings = array(
				'textarea_name' => 'wpgp_settings_options[wpgp_email_notifications_template_admin]',
				'media_buttons' => true,
				'textarea_rows' => 10,
				'tinymce'       => true,
				'quicktags'     => true,
			);

			// Display the editor
			wp_editor($email_notifications_template_user, 'wpgp_email_notifications_template_admin', $editor_settings); ?>
			
			<?php echo esc_html__('Use', 'gopray'); ?> <code>{{prayer_request_text}}</code> <?php echo esc_html__('to add the prayer text in admin mail notification', 'gopray'); ?>
		<?php
		}
		
		public function wpgp_enable_captcha_render() {
			$options = get_option('wpgp_settings_options', array());
			$recaptcha_type = isset($options['wpgp_recaptcha_type']) ? $options['wpgp_recaptcha_type'] : 'disable';
			?>
			<select name='wpgp_settings_options[wpgp_recaptcha_type]' id='wpgp_recaptcha_type' class="regular-text">
				<option value='disable' <?php selected($recaptcha_type, 'disable'); ?>><?php echo esc_html__('Disable', 'gopray'); ?></option>
				<option value='v2' <?php selected($recaptcha_type, 'v2'); ?>><?php echo esc_html__('reCAPTCHA v2', 'gopray'); ?></option>
				<option value='v3' <?php selected($recaptcha_type, 'v3'); ?>><?php echo esc_html__('reCAPTCHA v3', 'gopray'); ?></option>
			</select>
			<?php
		}
		
		public function wpgp_recaptcha_site_key_v2_render() {
			$options = get_option('wpgp_settings_options', array());
			$site_key = isset($options['wpgp_recaptcha_site_key_v2']) ? $options['wpgp_recaptcha_site_key_v2'] : '';
			?>
			<input type='text' name='wpgp_settings_options[wpgp_recaptcha_site_key_v2]' value='<?php echo esc_attr($site_key); ?>' id='wpgp_recaptcha_site_key_v2' class='regular-text'>
			<?php
		}

		public function wpgp_recaptcha_secret_key_v2_render() {
			$options = get_option('wpgp_settings_options', array());
			$secret_key = isset($options['wpgp_recaptcha_secret_key_v2']) ? $options['wpgp_recaptcha_secret_key_v2'] : '';
			?>
			<input type='text' name='wpgp_settings_options[wpgp_recaptcha_secret_key_v2]' value='<?php echo esc_attr($secret_key); ?>' id='wpgp_recaptcha_secret_key_v2' class='regular-text'>
			<?php
		}
		
		public function wpgp_recaptcha_site_key_v3_render() {
			$options = get_option('wpgp_settings_options', array());
			$site_key = isset($options['wpgp_recaptcha_site_key_v3']) ? $options['wpgp_recaptcha_site_key_v3'] : '';
			?>
			<input type='text' name='wpgp_settings_options[wpgp_recaptcha_site_key_v3]' value='<?php echo esc_attr($site_key); ?>' id='wpgp_recaptcha_site_key_v3' class='regular-text'>
			<?php
		}

		public function wpgp_recaptcha_secret_key_v3_render() {
			$options = get_option('wpgp_settings_options', array());
			$secret_key = isset($options['wpgp_recaptcha_secret_key_v3']) ? $options['wpgp_recaptcha_secret_key_v3'] : '';
			?>
			<input type='text' name='wpgp_settings_options[wpgp_recaptcha_secret_key_v3]' value='<?php echo esc_attr($secret_key); ?>' id='wpgp_recaptcha_secret_key_v3' class='regular-text'>
			<?php
		}
		
		public function wpgp_restrict_access_render() {
			$options = get_option('wpgp_settings_options', array());
			?>
<input type='checkbox' name='wpgp_settings_options[wpgp_restrict_access]' <?php checked(isset($options['wpgp_restrict_access']) ? $options['wpgp_restrict_access'] : 0, 1); ?> value='1'>
			<?php
		}
		
		public function wpgp_submit_color_render(){
			$options = get_option('wpgp_settings_options');
			$send_button_color = isset($options['wpgp_send_button_color']) ? $options['wpgp_send_button_color'] : '#000000';
			?>
			<input type='text' name='wpgp_settings_options[wpgp_send_button_color]' value='<?php echo esc_attr($send_button_color); ?>' class='wp-color-picker-field' id='send_button_color' placeholder="#000000" data-default-color='#000000' />
			<?php
		}
		
		public function wpgp_mail_with_smtp_render(){
			$options = get_option('wpgp_settings_options', array());
			?>
			<input type='checkbox' name='wpgp_settings_options[wpgp_mail_with_smtp]' <?php checked(isset($options['wpgp_mail_with_smtp']) ? $options['wpgp_mail_with_smtp'] : 0, 1); ?> value='1'>
			<?php
		}
		
		public function wpgp_enable_show_date_render() {
			$options = get_option('wpgp_settings_options', array());
            ?>
            <input type='checkbox' name='wpgp_settings_options[wpgp_enable_show_date]' <?php checked(isset($options['wpgp_enable_show_date']) ? $options['wpgp_enable_show_date'] : 0, 1); ?> value='1'>
            <?php
		}
		
		public function wpgp_enable_comments_render() {
            $options = get_option('wpgp_settings_options', array());
            ?>
            <input type='checkbox' name='wpgp_settings_options[wpgp_enable_comments]' <?php checked(isset($options['wpgp_enable_comments']) ? $options['wpgp_enable_comments'] : 0, 1); ?> value='1'>
            <?php
        }

        public function wpgp_enable_hearts_render() {
            $options = get_option('wpgp_settings_options', array());
            ?>
			<input type='checkbox' name='wpgp_settings_options[wpgp_enable_hearts]' <?php checked(isset($options['wpgp_enable_hearts']) ? $options['wpgp_enable_hearts'] : 0, 1); ?> value='1'>
            <?php
        }

		public function wpgp_display_period_render() {
			$options = get_option('wpgp_settings_options');
			$display_period = isset($options['wpgp_display_period']) ? $options['wpgp_display_period'] : '14';
			?>
			<select name='wpgp_settings_options[wpgp_display_period]' class="regular-text">
				<option value='all' <?php selected($display_period, 'all'); ?>><?php echo esc_html__('All', 'gopray'); ?></option>
				<option value='14' <?php selected($display_period, '14'); ?>><?php echo esc_html__('14 Days', 'gopray'); ?></option>
				<option value='30' <?php selected($display_period, '30'); ?>><?php echo esc_html__('30 Days', 'gopray'); ?></option>
				<option value='60' <?php selected($display_period, '60'); ?>><?php echo esc_html__('60 Days', 'gopray'); ?></option>
				<option value='90' <?php selected($display_period, '90'); ?>><?php echo esc_html__('90 Days', 'gopray'); ?></option>
			</select>
			<?php
		}
		
		public function wpgp_reply_color_render(){
			$options = get_option('wpgp_settings_options');
			$reply_button_color = isset($options['wpgp_reply_button_color']) ? $options['wpgp_reply_button_color'] : '#000000';
			?>
			<input type='text' name='wpgp_settings_options[wpgp_reply_button_color]' value='<?php echo esc_attr($reply_button_color); ?>' class='wp-color-picker-field' id='wpgp_reply_button_color' placeholder="#000000" data-default-color='#000000' />
			<?php
		}
		
		public function wpgp_load_color_render(){
			$options = get_option('wpgp_settings_options');
			$load_more_button_color = isset($options['wpgp_load_more_button_color']) ? $options['wpgp_load_more_button_color'] : '#000000';
			?>
			<input type='text' name='wpgp_settings_options[wpgp_load_more_button_color]' value='<?php echo esc_attr($load_more_button_color); ?>' class='wp-color-picker-field' id='load_more_button_color' placeholder="#000000" data-default-color='#000000' />
			<?php
		}
		
    }
	
	new WPGP_Prayer_Settings();
}