<?php
if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly
if (!class_exists('WPGP_Prayer_Shortcode')) {

    class WPGP_Prayer_Shortcode {
        
        public function __construct() {
			add_action('wp_enqueue_scripts', array($this, 'wpgp_enqueue_shortcode_assets'));
            add_shortcode('wpgp_prayer_request_form', array($this, 'wpgp_prayer_request_form_content'));
			add_shortcode('wpgp_display_prayer_wall', array($this, 'wpgp_display_prayer_wall_content'));
			add_filter('wp_mail_from', array($this, 'wpgp_mail_from'));
			add_filter('wp_mail_from_name', array($this, 'wpgp_mail_from_name'));
			
			// Handle prayer submission
			add_action('wp_ajax_submit_prayer_request', array($this, 'wpgp_ajax_submit_request_callback'));
			add_action('wp_ajax_nopriv_submit_prayer_request', array($this, 'wpgp_ajax_submit_request_callback'));
			
			// Handle comment submission
			add_action('wp_ajax_submit_prayer_comment', array($this, 'wpgp_ajax_submit_comment_callback'));
			add_action('wp_ajax_nopriv_submit_prayer_comment', array($this, 'wpgp_ajax_submit_comment_callback'));
			
			// Handle load more prayer
			add_action('wp_ajax_load_more_prayer_request', array($this, 'wpgp_ajax_load_more_prayer_request_callback'));
			add_action('wp_ajax_nopriv_load_more_prayer_request', array($this, 'wpgp_ajax_load_more_prayer_request_callback'));
			
			// Handle likes        
			add_action('wp_ajax_like_prayer_comment', array($this, 'wpgp_ajax_like_prayer_comment_callback'));
			add_action('wp_ajax_nopriv_like_prayer_comment', array($this, 'wpgp_ajax_like_prayer_comment_callback'));
		}
		
		
		public static function wpgp_prayer_request_form_content($atts) {
			ob_start();
			
			$wpgp_options = get_option('wpgp_settings_options', array());
			
            echo '<section class="wpgp-fullwidth-wrapper">';
			
			$args = [
				'wpgp_options' => $wpgp_options,
			];
		
			// load prayer form
			load_template(WPGP_PLUGIN_DIR . 'templates/prayer-from.php', '', $args);
			
			echo '</section>';
			
			return ob_get_clean();
        }
		
		
		public static function wpgp_display_prayer_wall_content($atts) {
			ob_start();
			
			$wpgp_options = get_option('wpgp_settings_options', array());
			
			$args = [
				'wpgp_options' => $wpgp_options,
			];
			
			echo '<section class="wpgp-fullwidth-wrapper">
			<!-- Container Starts here to wrap all comments -->
			<div class="wpgp-allcommentor-wrapper">';
			
			// load prayer wall
			load_template(WPGP_PLUGIN_DIR . 'templates/prayer-wall.php', '', $args);
			
			echo '
			</div>
        	<!-- Container ends here.  -->
			</section>';
			
			return ob_get_clean();
        }
		
		
		public static function wpgp_enqueue_shortcode_assets() {
			global $post;

			if(is_page() || is_single()) {
								
				$wpgp_options = get_option('wpgp_settings_options', array());
				$send_button_color = isset($wpgp_options['wpgp_send_button_color']) ? $wpgp_options['wpgp_send_button_color'] : '#19d5d9';
				$reply_button_color = isset($wpgp_options['wpgp_reply_button_color']) ? $wpgp_options['wpgp_reply_button_color'] : '#19d5d9';
				$load_more_button_color = isset($wpgp_options['wpgp_load_more_button_color']) ? $wpgp_options['wpgp_load_more_button_color'] : '#1DA1F2';
				$wpgp_css_variables = ':root {
					--wpgp-send-button-color: '. esc_html($send_button_color) .';
					--wpgp-reply-button-color: '. esc_html($reply_button_color) .';
					--wpgp-load-more-button-color: '. esc_html($load_more_button_color) .';
				}';
			
				
				$wpgp_js_variables = array(
					'ajax_url' => admin_url( 'admin-ajax.php' ),
					'wpgp_verification_nonce' => wp_create_nonce( 'wpgp_verification_nonce' ),
					'wpgp_fail_error' => __('Something went wrong. Please try again later', 'gopray'),
					'wpgp_reply_form' => '<div class="wpgp-commentor-replyform-wrapper">
						<form method="POST" class="wpgp-commentor-replyform">
							<textarea type="text" class="wpgp-replyform-texarea" name="wpgp_prayer_reqtext" placeholder="' . __('Comment', 'gopray') . '"></textarea>
							<label class="wpgp-replyform-labelguest">' . __('or as guest', 'gopray') . '</label><br /><br>
							<label class="wpgp-replyform-labelname">' . __('Name *', 'gopray') . '</label><br />
							<input type="text" class="wpgp-replyform-inputcontrol-single" name="wpgp_commentor_inpname" required />
							<div class="wpgp-replyform-replybtn-wrapper">
								<label class="wpgp-replyform-replyerlabel">' . __('Reply to', 'gopray') . '
									<span class="wpgp-replyform-replyer-namespan" id="wpgp-replyform-replyto-namespan"></span>
									<svg id="wpgp-replyform-closeID-formicon" aria-hidden="true" focusable="false" data-prefix="fas" data-icon="times" class="svg-inline--fa fa-times fa-w-11 fa-sm wpgp-replyform-replyer-closeicon" role="img" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 352 512">
										<path fill="#B6C1C6" d="M242.72 256l100.07-100.07c12.28-12.28 12.28-32.19 0-44.48l-22.24-22.24c-12.28-12.28-32.19-12.28-44.48 0L176 189.28 75.93 89.21c-12.28-12.28-32.19-12.28-44.48 0L9.21 111.45c-12.28 12.28-12.28 32.19 0 44.48L109.28 256 9.21 356.07c-12.28 12.28-12.28 32.19 0 44.48l22.24 22.24c12.28 12.28 32.2 12.28 44.48 0L176 322.72l100.07 100.07c12.28 12.28 32.2 12.28 44.48 0l22.24-22.24c12.28-12.28 12.28-32.19 0-44.48L242.72 256z"></path>
									</svg>
								</label><br />
								<input type="hidden" class="wpgp-replyform-inputattr" name="commentor_reply_id" value="" />
								<button type="submit" id="submit" class="wpgp-replyform-submitbtn" name="wpgp-submit-btn">' . __('Reply', 'gopray') . '</button>
							</div>
						</form>
					</div>',
				);
				
				$recaptcha_type = isset($wpgp_options['wpgp_recaptcha_type']) ? $wpgp_options['wpgp_recaptcha_type'] : 'disable';
				$recaptcha_site_key = '';
				if($recaptcha_type == 'v3'){
					$recaptcha_site_key = isset($wpgp_options['wpgp_recaptcha_site_key_v3']) && !empty($wpgp_options['wpgp_recaptcha_site_key_v3']) ? $wpgp_options['wpgp_recaptcha_site_key_v3'] : '00';
				}
				
				$wpgp_js_variables['recaptcha_v2_enable'] = $recaptcha_type == 'v2' ? true : false;
				$wpgp_js_variables['recaptcha_v3_enable'] = $recaptcha_type == 'v3' ? true : false;
				
				wp_enqueue_style('wpgp-front', WPGP_PLUGIN_URL . 'assets/css/front.css' , null, WPGP_PLUGIN_VERSION);
				wp_enqueue_script('wpgp-front', WPGP_PLUGIN_URL . 'assets/js/front.js', array('jquery'), WPGP_PLUGIN_VERSION, true);
				wp_add_inline_style('wpgp-front', $wpgp_css_variables);
				wp_localize_script( 'wpgp-front', 'wpgp_vars', $wpgp_js_variables );
				
				if($recaptcha_type == 'v2'){
					wp_enqueue_script('wpgp-g-recaptcha-v2', 'https://www.google.com/recaptcha/api.js', array(), 'v2.0', false);
				} elseif($recaptcha_type == 'v3' && $recaptcha_site_key){
					wp_enqueue_script('wpgp-g-recaptcha-v3', 'https://www.google.com/recaptcha/api.js?render='.$recaptcha_site_key, array(), 'v3.0', false);
					wp_add_inline_script('wpgp-g-recaptcha-v3', '
						async function wpgp_updateRecaptchaToken() {
							return new Promise(function(resolve, reject) {
								grecaptcha.ready(function() {
									try{
										grecaptcha.execute("' . esc_js($recaptcha_site_key) . '", { action: "submit" })
											.then(function(token) {
												document.getElementById("g-recaptcha-response").value = token;
												resolve(token);
											}).catch(function(error) {
												reject(error);
											});

									} catch (error) {
										console.log(error);
										reject(error);
									}
								});
							});
						}
					');
				}
			}
		}
		
		
		public function wpgp_ajax_like_prayer_comment_callback(){
			
			$response = array();
			
			if ( ( !isset($_POST['security']) || !wp_verify_nonce( sanitize_text_field( wp_unslash($_POST['security']) ), 'wpgp_verification_nonce') ) && is_user_logged_in()) {
                $response['status'] = 'failed';
				$response['error'] = __('Nonce verification failed', 'gopray');
				wp_send_json($response);
				wp_die();
            }
			
			if(isset($_POST['comment_id']) && !empty($_POST['comment_id'])){
				$_id = sanitize_text_field( wp_unslash($_POST['comment_id']) );
				
				if(get_post_type($_id) == 'wpgp_prayer_request'){
					
					$post_meta = get_post_meta($_id, '_wpgp_prayer_request_metadata', true);
					$current_likes = $post_meta['_wpgp_prayer_request_likes'];
					$post_meta['_wpgp_prayer_request_liked_by'] = isset($post_meta['_wpgp_prayer_request_liked_by']) && is_array($post_meta['_wpgp_prayer_request_liked_by']) ? $post_meta['_wpgp_prayer_request_liked_by'] : array();	
					
					$current_user_ip = $this->wpgp_get_the_user_ip();
					
					if (in_array($current_user_ip, $post_meta['_wpgp_prayer_request_liked_by'])) {
						$index = array_search($current_user_ip, $post_meta['_wpgp_prayer_request_liked_by']);
						unset($post_meta['_wpgp_prayer_request_liked_by'][$index]);
						$post_meta['_wpgp_prayer_request_likes'] = --$current_likes;
						$response['status'] = 'success';
						$response['updated_likes'] = $current_likes;
					} else {
						$post_meta['_wpgp_prayer_request_liked_by'][] = $current_user_ip;
						$post_meta['_wpgp_prayer_request_likes'] = ++$current_likes;
						$response['status'] = 'success';
						$response['updated_likes'] = $current_likes;
					}
					
					update_post_meta($_id, '_wpgp_prayer_request_metadata', $post_meta);
					
				} elseif(get_post_type($_id) == 'wpgp_prayer_comment'){
					$post_meta = get_post_meta($_id, '_wpgp_prayer_comment_metadata', true);
					$current_likes = intval($post_meta['_wpgp_prayer_comment_likes']);
					$post_meta['_wpgp_prayer_comment_liked_by'] = isset($post_meta['_wpgp_prayer_comment_liked_by']) && is_array($post_meta['_wpgp_prayer_comment_liked_by']) ? $post_meta['_wpgp_prayer_comment_liked_by'] : array();
					
					$current_user_ip = $this->wpgp_get_the_user_ip();
					
					if (in_array($current_user_ip, $post_meta['_wpgp_prayer_comment_liked_by'])) {
						$index = array_search($current_user_ip, $post_meta['_wpgp_prayer_comment_liked_by']);
						unset($post_meta['_wpgp_prayer_comment_liked_by'][$index]);
						$post_meta['_wpgp_prayer_comment_likes'] = --$current_likes;
						$response['status'] = 'success';
						$response['updated_likes'] = $current_likes;
					} else {
						$post_meta['_wpgp_prayer_comment_liked_by'][] = $current_user_ip;
						$post_meta['_wpgp_prayer_comment_likes'] = ++$current_likes;
						$response['status'] = 'success';
						$response['updated_likes'] = $current_likes;
					}
					
					update_post_meta($_id, '_wpgp_prayer_comment_metadata', $post_meta);
				}
			} else {
				$response['status'] = 'failed';
				$response['error'] = __('Comment id not found', 'gopray');
			}
			
			wp_send_json($response);
			die();
			
		}
		
		
		public function wpgp_ajax_load_more_prayer_request_callback(){
			$response = array();
			if ( ( !isset($_POST['security']) || !wp_verify_nonce(sanitize_text_field( wp_unslash($_POST['security']) ), 'wpgp_verification_nonce') ) && is_user_logged_in()) {
				$response['status'] = false;
				$response['error'] = __('Nonce verification failed', 'gopray');
				wp_send_json($response);
				die();
			}
			
			$wpgp_options = get_option('wpgp_settings_options', array());
			$paged = isset($_POST['paged']) && !empty($_POST['paged']) ? sanitize_text_field( wp_unslash($_POST['paged']) ) : 1;
				
			$args = [
				'wpgp_options' => $wpgp_options,
				'paged'		  => $paged,
			];

			// load more prayer request on prayer wall
			$response['status'] = true;
			ob_start();
			load_template(WPGP_PLUGIN_DIR . 'templates/prayer-wall.php', false, $args);
			$response['comments'] = ob_get_clean();
			wp_send_json($response);
			die();
		}
		
		
		public function wpgp_ajax_submit_request_callback() {

			$ResponseData = [];
			
            if ( ( !isset($_POST['security']) || !wp_verify_nonce(sanitize_text_field( wp_unslash($_POST['security']) ), 'wpgp_verification_nonce') ) && is_user_logged_in()) {
                $ResponseData['success'] = false;
				$ResponseData['error'] = __('Nonce verification failed', 'gopray');
				wp_send_json($ResponseData);
				wp_die();
            }
			
			$wpgp_options = get_option('wpgp_settings_options', array());
			$recaptcha_type = isset($wpgp_options['wpgp_recaptcha_type']) ? $wpgp_options['wpgp_recaptcha_type'] : 'disable';

			if ($recaptcha_type != 'disable') {
				if (isset($_POST['g-recaptcha-response']) && !empty($_POST['g-recaptcha-response'])) {
					$recaptcha_secret_key = '';

					if ($recaptcha_type == 'v2') {
						$recaptcha_secret_key = isset($wpgp_options['wpgp_recaptcha_secret_key_v2']) ? $wpgp_options['wpgp_recaptcha_secret_key_v2'] : '';
					} elseif ($recaptcha_type == 'v3') {
						$recaptcha_secret_key = isset($wpgp_options['wpgp_recaptcha_secret_key_v3']) ? $wpgp_options['wpgp_recaptcha_secret_key_v3'] : '';
					}

					if (!empty($recaptcha_secret_key)) {
						$recaptcha_response = sanitize_text_field( wp_unslash($_POST['g-recaptcha-response']) );
						
						$captcha_verification = wp_remote_get("https://www.google.com/recaptcha/api/siteverify?secret={$recaptcha_secret_key}&response={$recaptcha_response}");

						if (is_wp_error($captcha_verification)) {
							$ResponseData['success'] = false;
							$ResponseData['error'] = __('reCAPTCHA verification failed due to a connection error', 'gopray');
							wp_send_json($ResponseData);
							wp_die();
						}

						$captcha_keys = json_decode(wp_remote_retrieve_body($captcha_verification), true);

						if (!$captcha_keys["success"]) {
							$ResponseData['success'] = false;
							$ResponseData['error'] = __('reCAPTCHA verification failed', 'gopray');
							wp_send_json($ResponseData);
							wp_die();
						}
					} else {
						$ResponseData['success'] = false;
						$ResponseData['error'] = __('Missing reCAPTCHA secret key', 'gopray');
						wp_send_json($ResponseData);
						wp_die();
					}

				} else {
					$ResponseData['success'] = false;
					$ResponseData['error'] = __('Please verify the reCAPTCHA', 'gopray');
					wp_send_json($ResponseData);
					wp_die();
				}
			}
			
			$current_user_ip = $this->wpgp_get_the_user_ip();
			
			if (isset($_POST['_wpgp_prayer_request_formdata']) && !empty($_POST['_wpgp_prayer_request_formdata'])) {

					$pray = isset($_POST['_wpgp_prayer_request_formdata']['_wpgp_prayer_request_text']) ? sanitize_text_field( wp_unslash($_POST['_wpgp_prayer_request_formdata']['_wpgp_prayer_request_text']) ) : '';
					$prayer_name = isset($_POST['_wpgp_prayer_request_formdata']['_wpgp_prayer_request_name']) ? sanitize_text_field( wp_unslash($_POST['_wpgp_prayer_request_formdata']['_wpgp_prayer_request_name']) ) : '';
					$prayer_mail = isset($_POST['_wpgp_prayer_request_formdata']['_wpgp_prayer_request_email']) ? sanitize_email( wp_unslash($_POST['_wpgp_prayer_request_formdata']['_wpgp_prayer_request_email']) ) : '';

					if (!empty($pray)) {
						$pray_id = wp_insert_post(array(
							'post_title'    => wp_strip_all_tags($pray),
							'post_type'     => 'wpgp_prayer_request',
							'post_status'   => 'publish',
						));

						if (!is_wp_error($pray_id)) {
							
							wp_update_post(array(
								'ID'          => $pray_id,
								'post_status' => 'pending',
							));
							
							$meta_data = array(
								'_wpgp_prayer_request_name' =>  $prayer_name,
								'_wpgp_prayer_request_email' => $prayer_mail,
								'_wpgp_prayer_request_date' => get_the_date('Y-m-d', $pray_id),
								'_wpgp_prayer_request_ip_address' => sanitize_text_field($current_user_ip), // Assuming $current_user_ip is defined
								'_wpgp_prayer_request_likes' => 0,
								'_wpgp_prayer_request_related_comments' => array(),
							);

							update_post_meta($pray_id, '_wpgp_prayer_request_metadata', $meta_data);
							
							$user_notification_enabled = isset($wpgp_options['wpgp_email_notifications_user']) ? $wpgp_options['wpgp_email_notifications_user'] : 0;
							$admin_notification_enabled = isset($wpgp_options['wpgp_email_notifications_admin']) ? $wpgp_options['wpgp_email_notifications_admin'] : 0;
							$user_template = isset($wpgp_options['wpgp_email_notifications_template_user']) ? $wpgp_options['wpgp_email_notifications_template_user'] : '';
							$admin_template = isset($wpgp_options['wpgp_email_notifications_template_admin']) ? $wpgp_options['wpgp_email_notifications_template_admin'] : '';
							
							if($user_notification_enabled){
								$this->wpgp_user_mail_notification_prayer_submitted($pray_id , $user_template, $use_smtp);
							}
							
							if($admin_notification_enabled){
								$this->wpgp_admin_mail_notification_prayer_submitted($pray_id , $admin_template, $use_smtp, wp_strip_all_tags($pray));
							}

							$ResponseData['success'] = true;
							$ResponseData['message'] = __('Comment will be shown once reviewed by moderator', 'gopray');
						} else {
							$ResponseData['success'] = false;
							$ResponseData['error'] = __('Pray submission failed due to internal error', 'gopray');
						}
					} else {
						$ResponseData['success'] = false;
						$ResponseData['error'] = __('Comment cannot be empty', 'gopray');
					}
			} else {
				$ResponseData['success'] = false;
				$ResponseData['error'] = __('Invalid form data', 'gopray');
			}
        
            wp_send_json($ResponseData);
			wp_die();
        }
		
		
		public function wpgp_ajax_submit_comment_callback() {

			$ResponseData = [];
			
            if ( ( !isset($_POST['security']) || !wp_verify_nonce(sanitize_text_field( wp_unslash($_POST['security']) ), 'wpgp_verification_nonce') ) && is_user_logged_in()) {
                $ResponseData['success'] = false;
				$ResponseData['error'] = __('Nonce verification failed', 'gopray');
				wp_send_json($ResponseData);
				wp_die();
            }
			
			$current_user_ip = $this->wpgp_get_the_user_ip();
			
			if(isset($_POST['commentor_reply_id']) && !empty($_POST['commentor_reply_id'])){
				
				if( isset($_POST['wpgp_prayer_reqtext']) && !empty($_POST['wpgp_prayer_reqtext']) ){
					
					$comment = isset($_POST['wpgp_prayer_reqtext']) ? sanitize_text_field( wp_unslash($_POST['wpgp_prayer_reqtext']) ) : '';
					$commentor_name = isset($_POST['wpgp_commentor_inpname']) ? sanitize_text_field( wp_unslash($_POST['wpgp_commentor_inpname']) ) : '';
					$reply_on_post_id = intval($_POST['commentor_reply_id']);
					$commentor_mail = '';
					$reply_on_post_type = get_post_type($reply_on_post_id);

					$comment_id = wp_insert_post(array(
						'post_title'    => wp_strip_all_tags($comment),
						'post_type'     => 'wpgp_prayer_comment',
						'post_status'   => 'pending',             
					));

					if (!is_wp_error($comment_id)) {

						$meta_data['_wpgp_prayer_comment_name'] = $commentor_name;
						$meta_data['_wpgp_prayer_comment_email'] = $commentor_mail;
						$meta_data['_wpgp_prayer_comment_date'] = get_the_date('Y-m-d', $comment_id);
						$meta_data['_wpgp_prayer_comment_ip_address'] = $current_user_ip;
						$meta_data['_wpgp_prayer_comment_likes'] = 0;

						if($reply_on_post_type == 'wpgp_prayer_request'){
							// Add comment in prayer's related comments
							$parent_prayer_request_metadata = get_post_meta($reply_on_post_id, '_wpgp_prayer_request_metadata', true);
							$parent_prayer_request_metadata['_wpgp_prayer_request_related_comments'][] = $comment_id;
							update_post_meta($reply_on_post_id, '_wpgp_prayer_request_metadata', $parent_prayer_request_metadata);

							$meta_data['_wpgp_parent_request_id'] = $reply_on_post_id;

						} elseif($reply_on_post_type == 'wpgp_prayer_comment'){
							// Add comment in comment's related sub-comments
							$parent_prayer_comment_metadata = get_post_meta($reply_on_post_id, '_wpgp_prayer_comment_metadata', true);

							if(isset($parent_prayer_comment_metadata['_wpgp_comment_related_sub_comments']) && is_array($parent_prayer_comment_metadata['_wpgp_comment_related_sub_comments'])){
								$parent_prayer_comment_metadata['_wpgp_comment_related_sub_comments'][] = $comment_id;
							} else {
								$parent_prayer_comment_metadata['_wpgp_comment_related_sub_comments'] = array($comment_id);
							}
							
							update_post_meta($reply_on_post_id, '_wpgp_prayer_comment_metadata', $parent_prayer_comment_metadata);
							
							$meta_data['_wpgp_parent_request_id'] = $reply_on_post_id;

						}

						update_post_meta($comment_id, '_wpgp_prayer_comment_metadata', $meta_data);

						$ResponseData['success'] = true;
						$ResponseData['message'] = __('Reply comment will be shown once reviewed by moderator', 'gopray');
					} else {
						$ResponseData['success'] = false;
						$ResponseData['error'] = __('Comment submission failed due to internal error', 'gopray');
					}
				} else {
					$ResponseData['success'] = false;
					$ResponseData['error'] = __('Comment cannot be empty', 'gopray');
				}

			} else {
				$ResponseData['success'] = false;
				$ResponseData['error'] = __('Prayer id not found', 'gopray');
			}
        
            wp_send_json($ResponseData);
			wp_die();
        }
		
		
		public static function wpgp_user_mail_notification_prayer_submitted($prayer_id, $user_mail_template) {

			$prayer_request_meta = get_post_meta($prayer_id, '_wpgp_prayer_request_metadata', true);
			$user_email = isset($prayer_request_meta['_wpgp_prayer_request_email']) ? $prayer_request_meta['_wpgp_prayer_request_email'] : '';

			if ($user_email) {

				$user_subject = __('Your Prayer Request has been Submitted', 'gopray');
				$headers = array('Content-Type: text/html; charset=UTF-8');
				
				$user_mail_template = str_replace('\\', '', $user_mail_template);

				// Send email
				wp_mail($user_email, $user_subject, $user_mail_template, $headers);

			}
		}
		
		
		public static function wpgp_admin_mail_notification_prayer_submitted($prayer_id, $admin_mail_template, $send_with_smtp, $prya_text) {
			
			$admin_email = get_option( 'admin_email' );
			if ($admin_email) {

				$admin_subject = __('New Prayer Request Submission', 'gopray');
				$headers = array('Content-Type: text/html; charset=UTF-8');
				
				$mail_body =  str_replace("{{prayer_request_text}}", $prya_text, $admin_mail_template);
				$mail_body = str_replace('\\', '', $mail_body);

				// Send email
				wp_mail($admin_email, $admin_subject, $mail_body, $headers);

			}
		}
		
		
		public static function wpgp_mail_from($email) {
			$wpgp_options = get_option('wpgp_settings_options', array());
			$use_smtp = isset($wpgp_options['wpgp_mail_with_smtp']) ? $wpgp_options['wpgp_mail_with_smtp'] : 0;
			
			$host = isset( $_SERVER['HTTP_HOST'] ) ? sanitize_text_field( wp_unslash( $_SERVER['HTTP_HOST'] ) ) : '';
			if ( !empty( $host ) && !$use_smtp ) {
				$parts = explode('.', $host);
				$tld = array_pop($parts);
				$domain = implode('.', $parts);
				$email = "{$domain}@{$host}";
			}
			
			return $email;
		}
		
		public static function wpgp_mail_from_name($name) {
			$wpgp_options = get_option('wpgp_settings_options', array());
			$use_smtp = isset($wpgp_options['wpgp_mail_with_smtp']) ? $wpgp_options['wpgp_mail_with_smtp'] : 0;
			
			$host = isset( $_SERVER['HTTP_HOST'] ) ? sanitize_text_field( wp_unslash( $_SERVER['HTTP_HOST'] ) ) : '';
			if (!empty( $host ) && !$use_smtp) {
				$parts = explode('.', $host);
				$tld = array_pop($parts);
				$name = implode('.', $parts);
			}
			
			return $name;
		}
		
		
		public static function wpgp_get_the_user_ip() {

			if ( ! empty( $_SERVER['HTTP_CLIENT_IP'] ) ) {
				$ip = sanitize_text_field( wp_unslash($_SERVER['HTTP_CLIENT_IP']) );
			} elseif ( ! empty( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ) {
				$ip = sanitize_text_field( wp_unslash($_SERVER['HTTP_X_FORWARDED_FOR']) );
			} elseif ( ! empty( $_SERVER['REMOTE_ADDR'] ) ) {
				$ip = sanitize_text_field( wp_unslash($_SERVER['REMOTE_ADDR']) );
			}
			
			// Handle localhost IP addresses
			if ( $ip === '::1' || $ip === '127.0.0.1' ) {
				$ip = '127.0.0.1';
			}
			
			return sanitize_text_field( $ip );
		}
		
    }
    
    new WPGP_Prayer_Shortcode();

}